package org.kantega.jexmec.groovy;

/*
 * Copyright 2011 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.kantega.jexmec.ctor.ClassLocator;
import org.kantega.jexmec.ctor.ConstructorInjectionPluginLoader;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 */
public class GroovyPluginLoader<P> extends ConstructorInjectionPluginLoader<P> {


    @Override
    protected ClassLocator<Class<P>> createPluginClassLocator(Class<P> pluginClass) {
        return new GroovyClassLocator(pluginClass);
    }

    public void unloadPlugins(ClassLoader classLoader) {
    }

    public void start() {

    }

    public void stop() {
    }

    private class GroovyClassLocator extends ClassLocator<Class<P>> {
        private final Class<P> pluginClass;

        public GroovyClassLocator(Class<P> pluginClass) {
            this.pluginClass = pluginClass;
        }

        @Override
        public List<Class<P>> locateClasses(ClassLoader classLoader) {
            if (classLoader instanceof JexmecGroovyClassLoader) {
                JexmecGroovyClassLoader loader = (JexmecGroovyClassLoader) classLoader;

                List<Class<P>> pluginClasses = new ArrayList<Class<P>>();

                for(Class clazz : loader.getClasses()) {
                    if(pluginClass.isAssignableFrom(clazz)) {
                        pluginClasses.add(clazz);
                    }
                }
                return pluginClasses;
            } else {
                return Collections.emptyList();
            }

        }
    }
}
